package com.alterdekim.flash.decompiler.translator;

import com.alterdekim.flash.decompiler.action.*;
import com.alterdekim.flash.decompiler.item.*;
import com.alterdekim.flash.decompiler.item.internal.InitObjectItem;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.harmony.unpack200.bytecode.ByteCode;

import java.util.*;

@Slf4j
@Getter
public class Java2Flash {

    private List<String> pool;
    private List<ByteCodeAction> actions;

    public Java2Flash convert(Map<String, Object> map) {
        this.actions = new ArrayList<>();
        this.pool = new ArrayList<>();
        this.pool.addAll(extractAllStrings(new HashSet<>(65535), map)); // maybe make a check if hashset size is bigger than initialCapacity
        this.actions.add(new ActionConstantPool(this.pool));
        map.keySet().forEach(k -> createVariable(map, k));
        map.keySet().stream().filter(k -> map.get(k).getClass().getSimpleName().equals("HashMap")).forEach(k -> setMembers(map, k));
        return this;
    }

    private void setMembers(Map<String, Object> map, String key) {
        HashMap<String, Object> m = (HashMap<String, Object>) map.get(key);
        m.keySet().forEach(k -> {
            this.actions.add(new ActionPush(Collections.singletonList(fromString(key))));
            this.actions.add(new ActionGetVariable());
            this.actions.add(new ActionPush(switch (m.get(k).getClass().getSimpleName()) {
                case "String" -> List.of(fromString(k), fromString((String) m.get(k)));
                case "Integer" -> List.of(fromString(k), new IntegerItem((Integer) m.get(k)));
                case "Double" -> List.of(fromString(k), new DoubleItem((Double) m.get(k)));
                case "Boolean" -> List.of(fromString(k), new BooleanItem((Boolean) m.get(k)));
                default -> List.of(fromString(k));
            }));
            if( m.get(k).getClass().getSimpleName().equals("HashMap") ) {
                HashMap<String, Object> s = (HashMap<String, Object>) m.get(k);
                parseHashMapMember(s);
                this.actions.add(new ActionPush(Collections.singletonList(new IntegerItem(s.keySet().size()))));
                this.actions.add(new ActionInitObject());
            }
            this.actions.add(new ActionSetMember());
        });
    }

    private void parseHashMapMember(Map<String, Object> m) {
        m.keySet().forEach(key -> {
            switch (m.get(key).getClass().getSimpleName()) {
                case "String":
                    this.actions.add(new ActionPush(List.of(fromString(key), fromString((String) m.get(key)))));
                    break;
                case "Integer":
                    this.actions.add(new ActionPush(List.of(fromString(key), new IntegerItem((Integer) m.get(key)))));
                    break;
                case "Double":
                    this.actions.add(new ActionPush(List.of(fromString(key), new DoubleItem((Double) m.get(key)))));
                    break;
                case "Boolean":
                    this.actions.add(new ActionPush(List.of(fromString(key), new BooleanItem((Boolean) m.get(key)))));
                    break;
                case "HashMap":
                    HashMap<String, Object> s = (HashMap<String, Object>) m.get(key);
                    parseHashMapMember(s);
                    this.actions.add(new ActionPush(Collections.singletonList(new IntegerItem(s.keySet().size()))));
                    this.actions.add(new ActionInitObject());
                    break;
            }
        });
    }

    private void createVariable(Map<String, Object> map, String key) {
        List<ByteCodeItem> items = new ArrayList<>();
        items.add(fromString(key));
        log.info("Type: {}", map.get(key).getClass().getSimpleName());
        items.add(switch (map.get(key).getClass().getSimpleName()) {
            case "String" -> fromString((String) map.get(key));
            case "Integer" -> new IntegerItem((Integer) map.get(key));
            case "Double" -> new DoubleItem((Double) map.get(key));
            case "Boolean" -> new BooleanItem((Boolean) map.get(key));
            default -> new IntegerItem(0);
        });
        this.actions.add(new ActionPush(items));
        if( map.get(key).getClass().getSimpleName().equals("HashMap") ) {
            this.actions.add(new ActionInitObject());
        }
        this.actions.add(new ActionDefineLocal());
    }

    private ByteCodeItem fromString(String s) {
        int i = this.pool.indexOf(s);
        return i != -1 ? new PoolIndexItem(i) : new StringItem(s);
    }

    private Set<String> extractAllStrings(Set<String> pool, Map<String, Object> map) {
        map.keySet().forEach(k -> {
            try {
                Integer.parseInt(k);
            } catch (Exception e) {
                pool.add(k);
            }
        });
        map.values().forEach(o -> {
            switch (o.getClass().getSimpleName()) {
                case "HashMap" -> extractAllStrings(pool, (HashMap<String, Object>) o);
                case "String" -> pool.add((String) o);
            }
        });
        return pool;
    }
}